#!/bin/bash

#================================================================
# HEADER
#================================================================
#% SYNOPSIS
#%
#================================================================
#- IMPLEMENTATION
#-    version         1.0.6
#-    author          Artem Senko
#-    copyright       Copyright (c) http://www.identite.us
#-
#================================================================

#-------------------Global Variables-----------------------------
docker_eng_ver=19.03.0
docker_compose_ver=1.24.0
global_var_file=variables.env
nopass_var_file=nopass.env
nopass_image=""
nopass_port=80
nopass_port_selection=""
is_exist_nopass=""
radius_auth_port=1812
radius_acco_port=1813
preshop_var_file=preshop.env
sbank_var_file=sbank.env
cportal_var_file=cportal.env
nginx_var_file=nginx.env
preshop_tmp_line=""
sbank_tmp_line=""
cportal_tmp_line=""
nginx_type=null
var_types=()
public_valid_name=0
dockerhub=hubdocker.identite.us
req_nopass=(
  "MAPD_DatabaseSettings__DbConnectionType"
  "MAPD_DatabaseSettings__Server"
  "MAPD_DatabaseSettings__Port"
  "MAPD_DatabaseSettings__DatabaseName"
  "MAPD_DatabaseSettings__UserId"
  "MAPD_DatabaseSettings__Password"
)
req_preshop=(
  "PRESHOP_DbType"
  "PRESHOP_ConnectionStrings__DefaultConnection"
  "PRESHOP_AfterLoginPage"
)

req_sbank=(
  "SBANK_DbType"
  "SBANK_ConnectionStrings__DefaultConnection"
  "SBANK_AfterLoginPage"
)

req_cportal=(
  "CPORTAL_Oidc__Authority"
)


#-------------------Functions------------------------------------
# NoPass banner
nopass_banner()
{
  echo "+---------------------------------------------------------------------------+"
  echo "		##    ##  #######  ########     ###     ######   ######  "
  echo "		###   ## ##     ## ##     ##   ## ##   ##    ## ##    ## "
  echo "		####  ## ##     ## ##     ##  ##   ##  ##       ##       "
  echo "		## ## ## ##     ## ########  ##     ##  ######   ######  "
  echo "		##  #### ##     ## ##        #########       ##       ## "
  echo "		##   ### ##     ## ##        ##     ## ##    ## ##    ## "
  echo "		##    ##  #######  ##        ##     ##  ######   ######  "
}

# Main menu banner
main_banner()
{
  echo "+---------------------------------------------------------------------------+"
  printf "| %-73s |\n" "`date`"
  echo "|                                                                           |"
  printf "|`tput bold` %-73s `tput sgr0`|\n" "$@"

  echo "+---------------------------------------------------------------------------+"
}

# Check root permissions
is_root()
{
if [ "$EUID" -ne 0 ]
  then
    echo "ERROR! Please run script as root. Script execution stopped"
  exit
fi
}

# Check if command exists
command_exists() {
	command -v "$@" > /dev/null 2>&1
}

# Check if nopass is deployed

check_exist_nopass() {
  if [ "$(docker ps -a | grep nopass)" ] > /dev/null 2>&1 ; then
    # General inroduction
    echo "+---------------------------------------------------------------------------+"
    echo "Information. The currently running NoPass version will be stopped"
    echo "Information. Do you want to continue?"
    echo "Please enter a valid input"
    echo "1.Yes"
    echo "2.No"
    # Install type
    read is_exist_nopass
    until [ "$is_exist_nopass" = 1 ] || [ "$is_exist_nopass" = 2 ]; do
      echo "Please enter a valid input"
      echo "1.Yes"
      echo "2.No"
      echo "You may press Ctrl+C now to abort this script."
      read is_exist_nopass
    done
    if [[ -f docker-compose.yml ]] && [[ "$is_exist_nopass" = 1 ]]; then
      echo "+---------------------------------------------------------------------------+"
      docker-compose down
    elif [[ ! -f docker-compose.yml ]] && [[ "$is_exist_nopass" = 1 ]]; then
      echo "+---------------------------------------------------------------------------+"
      docker rm $(docker stop $(docker ps -a -q --filter="name=nopass*"))
    else
      echo "Information. Script execution stopped. Please stop the current NoPass server"
      exit 1;
    fi
  fi
}

# Pre-install preparations
preparation ()
{
  echo "+---------------------------------------------------------------------------+"
  echo "Information. Old config files will be deleted"
  rm -f nginx.env
  rm -f nopass.env
  rm -f preshop.env
  rm -f sbank.env
  rm -f cportal.env
  rm -rf nginx
  rm -rf docker-compose.yml
  rm -rf logs
  rm -rf mysql
}

# Docker logib to Identite dockerhub
docker_login()
{
  echo "+---------------------------------------------------------------------------+"
  echo "Please enter username and password to login the Identite Docker hub"
  docker login $dockerhub || exit 1
  echo "+---------------------------------------------------------------------------+"
  echo "Information. Docker login to Identite Docker hub passed"
}

# Get current docker engine version
get_docker_engine_version()
{
  if command_exists docker && [ -e /var/run/docker.sock ]
  then
          echo $(docker --version | awk -F'[ ]' '{print substr($3, 1, length($3)-1)}')
  fi
}

# Get current docker-compose version
get_docker_compose_version()
{
  if command_exists docker-compose
  then
          echo $(docker-compose --version | awk -F'[ ]' '{print substr($3, 1, length($3)-1)}')
  fi
}

# Additional method for install

check_install_result()
{
  res=$1
  until [ "$res" = 1 ] || [ "$res" = 2 ]; do
    echo "Please enter a valid input"
    echo "1.Yes"
    echo "2.No"
    echo "You may press Ctrl+C now to abort this script."
    read res
  done
  return $res
}

# Get Docker CE

docker_engine_install()
{
  echo "Do you want to install Docker automatically?"
  echo "+---------------------------------------------------------------------------+"
  echo "Please select the option below:"
  echo "1.Yes"
  echo "2.No"
  echo "+---------------------------------------------------------------------------+"
  read result
  check_install_result $result
  if [ $? = 1 ]
  then
    echo "Information. Docker is being installed, please wait..."
    curl -fsSL https://get.docker.com | sh
    echo "Information. Docker installed successfully"
  else
    echo "+---------------------------------------------------------------------------+"
    echo "Script execution stopped"
    echo "Please see how to install Docker engine at https://docs.docker.com/install/ "
    echo "Restart the script when you install Docker engine"
    echo "+---------------------------------------------------------------------------+"
    exit 1
  fi
}

# Get Docker-compose tool

docker_compose_install()
{
  echo "Do you want to install Docker Compose tool automatically?"
  echo "+---------------------------------------------------------------------------+"
  echo "Please select the option below:"
  echo "1.Yes"
  echo "2.No"
  echo "+---------------------------------------------------------------------------+"
  read result
  check_install_result $result
  if [ $? = 1 ]
  then
    echo "Information. Docker Compose tool is being installed, please wait..."
    curl -L "https://github.com/docker/compose/releases/download/1.28.2/docker-compose-$(uname -s)-$(uname -m)" -o /usr/local/bin/docker-compose
    chmod +x /usr/local/bin/docker-compose
    ln -sf /usr/local/bin/docker-compose /usr/bin/docker-compose
    docker-compose --version
    echo "Information. Docker Compose tool installed successfully"
  else
    echo "+---------------------------------------------------------------------------+"
    echo "Script execution stopped"
    echo "Please see how to install Docker Compose at https://docs.docker.com/compose/install/ "
    echo "Restart the script when you install Docker Compose tool"
    echo "+---------------------------------------------------------------------------+"
    exit 1
  fi

}

# Compare current and required docker versions
version_check() {
#-------------------Local Variables-----------------------------
  local IFS=.
  local i ver1=($1) ver2=($2)

  if [[ $1 == $2 ]]
  then
      return 1
  fi
  # fill empty fields in ver1 with zeros
  for ((i=${#ver1[@]}; i<${#ver2[@]}; i++))
  do
      ver1[i]=0
  done
  for ((i=0; i<${#ver1[@]}; i++))
  do
      if [[ -z ${ver2[i]} ]]
      then
          # fill empty fields in ver2 with zeros
          ver2[i]=0
      fi
      if ((10#${ver1[i]} > 10#${ver2[i]}))
      then
          return 1
      fi
      if ((10#${ver1[i]} < 10#${ver2[i]}))
      then
          return 2
      fi
  done
  return 0
}

# Menu for docker installation

docker_menu() {
#-------------------Local Variables-----------------------------
  local docker_engine_current_ver=$(get_docker_engine_version)
  local docker_compose_current_ver=$(get_docker_compose_version)

  if ! [ -z $docker_engine_current_ver ]; then
    version_check $docker_engine_current_ver $docker_eng_ver
    case $? in
      1)
      echo "+---------------------------------------------------------------------------+"
      echo "Information. Docker version check passed";;
      2)
      echo "+---------------------------------------------------------------------------+"
      echo "Error. Required version is missing"
      docker_engine_install;;
    esac
  else
    	echo "+---------------------------------------------------------------------------+"
      echo "Error. Docker isn't installed"
      docker_engine_install
  fi

  if ! [ -z $docker_compose_current_ver ]; then
    version_check $docker_compose_current_ver $docker_compose_ver
    case $? in
      1)
      echo "+---------------------------------------------------------------------------+"
      echo "Information. Docker-compose version check passed";;
      2)
      echo "+---------------------------------------------------------------------------+"
      echo "Error. Required version is missing"
      docker_compose_install;;
    esac
  else
    echo "+---------------------------------------------------------------------------+"
    echo "Error. Docker Compose tool isn't installed"
    docker_compose_install
  fi
}

# Change radius ports

change_radius_port() {
  # General intro
  echo "+---------------------------------------------------------------------------+"
  echo "By default, Nopass container listens for RADIUS traffic on ports 1812/UDP and 1813/UDP"
  echo "Do you want to change these ports?"
  echo "+---------------------------------------------------------------------------+"
  echo "Please select the option below:"
  echo "1.Yes"
  echo "2.No"
  # Install type
  read is_change_radius_port
  until [ "$is_change_radius_port" = 1 ] || [ "$is_change_radius_port" = 2 ]; do
    echo "+---------------------------------------------------------------------------+"
    echo "Error. Please enter a valid input:"
    echo "1.Yes"
    echo "2.No"
    echo "You may press Ctrl+C now to abort this script."
    read is_change_radius_port
  done
  # Branching
  if [ "$is_change_radius_port" = 1 ]; then
    # Authentication intro
    echo "+---------------------------------------------------------------------------+"
    echo "Do you want to change authentication port? Default: 1812"
    echo "+---------------------------------------------------------------------------+"
    echo "Please select the option below:"
    echo "1.Yes"
    echo "2.No"
    # Install type
    read is_change_radius_auth
    until [ "$is_change_radius_auth" = 1 ] || [ "$is_change_radius_auth" = 2 ]; do
      echo "+---------------------------------------------------------------------------+"
      echo "Error. Please enter a valid input:"
      echo "1.Yes"
      echo "2.No"
      echo "You may press Ctrl+C now to abort this script."
      read is_change_radius_auth
    done
    # Branching
    if [ "$is_change_radius_auth" = 1 ]; then
      # Authentication port intro
      echo "+---------------------------------------------------------------------------+"
      echo "Please input a new one RADIUS authentication port. Default: 1812"
      echo "+---------------------------------------------------------------------------+"
      # Install type
      read radius_auth_port
      until [[ "$radius_auth_port" =~ ^[0-9]{0,5}$ ]]; do
        echo "+---------------------------------------------------------------------------+"
        echo "Error. Port should be contain no more than 5 digits. Please try again"
        echo "You may press Ctrl+C now to abort this script."
        read radius_auth_port
      done
    fi
    # Accounting intro
    echo "+---------------------------------------------------------------------------+"
    echo "Do you want to change accounting port? Default: 1813"
    echo "+---------------------------------------------------------------------------+"
    echo "Please select the option below:"
    echo "1.Yes"
    echo "2.No"
    # Install type
    read is_change_radius_acco
    until [ "$is_change_radius_acco" = 1 ] || [ "$is_change_radius_acco" = 2 ]; do
      echo "+---------------------------------------------------------------------------+"
      echo "Error. Please enter a valid input:"
      echo "1.Yes"
      echo "2.No"
      echo "You may press Ctrl+C now to abort this script."
      read is_change_radius_acco
    done
    # Branching
    if [ "$is_change_radius_acco" = 1 ]; then
      # Accounting port intro
      echo "+---------------------------------------------------------------------------+"
      echo "Please input a new one RADIUS accounting port. Default: 1813"
      echo "+---------------------------------------------------------------------------+"
      # Install type
      read radius_acco_port
      until [[ "$radius_acco_port" =~ ^[0-9]{0,5}$ ]]; do
        echo "+---------------------------------------------------------------------------+"
        echo "Error. Port should be contain no more than 5 digits. Please try again"
        echo "You may press Ctrl+C now to abort this script."
        read radius_acco_port
      done
    fi
  fi

}

# Change NoPass application port from default - 80

change_nopass_port() {
  # General intro
  echo "+---------------------------------------------------------------------------+"
  echo "The NoPass server uses the default listen port 80"
  echo "Do you want to change this port to any other?"
  echo "+---------------------------------------------------------------------------+"
  echo "Please select the option below:"
  echo "1.Yes"
  echo "2.No"
  # Install type
  read nopass_port_selection
  until [ "$nopass_port_selection" = 1 ] || [ "$nopass_port_selection" = 2 ]; do
    echo "+---------------------------------------------------------------------------+"
    echo "Error. Please enter a valid input:"
    echo "1.Yes"
    echo "2.No"
    echo "You may press Ctrl+C now to abort this script."
    read nopass_port_selection
  done
  # Branching
  if [ "$nopass_port_selection" = 1 ]; then
    # Authentication port intro
    echo "+---------------------------------------------------------------------------+"
    echo "Please input a new one NoPass port. Default: 80"
    echo "+---------------------------------------------------------------------------+"
    # Install type
    read nopass_port
    until [[ "$nopass_port" =~ ^[0-9]{0,5}$ ]]; do
      echo "+---------------------------------------------------------------------------+"
      echo "Error. Port should be contain no more than 5 digits. Please try again "
      echo "You may press Ctrl+C now to abort this script."
      read nopass_port
    done
  fi
}

main_menu()
{
  echo "+---------------------------------------------------------------------------+"
  echo "Do you want to install for Production or Demo purposes?"
  echo "Note. Demo purpose assumes additionally installation of a service with integrated"
  echo "API to NoPass connect"
  echo "+---------------------------------------------------------------------------+"
  echo "Please select the option below:"
  echo "1.Production"
  echo "2.Demo"
# Install type
  read install_type
  until [ "$install_type" = 1 ] || [ "$install_type" = 2 ]; do
    echo "+---------------------------------------------------------------------------+"
    echo "Error. Please enter a valid input:"
    echo "1.Production"
    echo "2.Demo"
    echo "You may press Ctrl+C now to abort this script."
    read install_type
  done
  if [ "$install_type" = 2 ]; then
    echo "+---------------------------------------------------------------------------+"
    echo "What service do you want to install?"
    echo "If you choose several services please enter it without space or comma"
    echo "e.g. Preshop and Securebank: 12"
    echo "+---------------------------------------------------------------------------+"
    echo "Please select the option below:"
    echo "1.Preshop"
    echo "2.Securebank"
    echo "3.Corporate portal"
    # Demo service type
      read demo_service
      until [ "$demo_service" = 1 ] || [ "$demo_service" = 2 ] || [ "$demo_service" = 3 ] || [ "$demo_service" = 12 ] || [ "$demo_service" = 23 ] || [ "$demo_service" = 13 ] || [ "$demo_service" = 123 ]; do
        echo "+---------------------------------------------------------------------------+"
        echo "Error. Please enter a valid input:"
        echo "1.Preshop"
        echo "2.Securebank"
        echo "3.Corporate portal"
        echo "You may press Ctrl+C now to abort this script."
        read demo_service
      done
  fi
    change_radius_port
  if [ "$install_type" = 1 ]; then
    echo "+---------------------------------------------------------------------------+"
    echo "For SSL termination you have to use a Reverse Proxy."
    echo "We suggest you set up a Nginx Reverse Proxy server automatically or"
    echo "you can install and configure server yourself."
    echo "+---------------------------------------------------------------------------+"
    echo "Do you want to install the Nginx Reverse Proxy automatically?"
    echo "+---------------------------------------------------------------------------+"
    echo "Please select the option below:"
    echo "1.Yes"
    echo "2.No"
    read nginx_type
    until [ "$nginx_type" = 1 ] || [ "$nginx_type" = 2 ]; do
      echo "+---------------------------------------------------------------------------+"
      echo "Error. Please enter a valid input:"
      echo "1.Yes"
      echo "2.No"
      echo "You may press Ctrl+C now to abort this script."
      read nginx_type
    done
  else
    nginx_type=1
  fi
# Nginx selection menu, get SSL public and private certificate/key
  if [ "$nginx_type" = 1 ]; then
    for var in $(ls *.{crt,key,cert,pem} 2>/dev/null); do
      if grep -E "PRIVATE" $var >/dev/null 2>&1; then
        key_ssl_name=$var
      elif grep -E "BEGIN CERTIFICATE" $var >/dev/null 2>&1; then
        public_ssl_name=$var
      fi
    done
    echo "+---------------------------------------------------------------------------+"
    echo "You should copy SSL certificates to the root directory."
    echo "Did you copy the certificates?"
    echo "+---------------------------------------------------------------------------+"
    echo "Please select the option below:"
    echo "1.Yes"
    echo "2.No"
    read cert_exists
    until [ "$cert_exists" = 1 ] || [ "$cert_exists" = 2 ]; do
      echo "+---------------------------------------------------------------------------+"
      echo "Error. Please enter a valid input:"
      echo "1.Yes"
      echo "2.No"
      echo "You may press Ctrl+C now to abort this script."
      read cert_exists
    done
    if [ "$cert_exists" = 1 ]; then
      echo "+---------------------------------------------------------------------------+"
      echo "To automatically generate configs you have to enter SSL file names"
      if ! [[ -z $public_ssl_name ]]; then
        echo "+---------------------------------------------------------------------------+"
        echo "Is $public_ssl_name the public certificate?"
        echo "1.Yes"
        echo "2.No"
        read public_valid_name
        until [ "$public_valid_name" = 1 ] || [ "$public_valid_name" = 2 ]; do
          echo "+---------------------------------------------------------------------------+"
          echo "Error. Please enter a valid input:"
          echo "1.Yes"
          echo "2.No"
          echo "You may press Ctrl+C now to abort this script."
          read public_valid_name
        done
      fi
      if [ "$public_valid_name" != 1 ]; then
        echo "Enter public certificate name:"
        echo "+---------------------------------------------------------------------------+"
        read public_ssl_name
        until ! [[ -z $public_ssl_name ]] && [[ -e $public_ssl_name ]]; do
          echo "+---------------------------------------------------------------------------+"
          echo "Error. No such file. Please enter a valid certificate name"
          echo "You may press Ctrl+C now to abort this script."
          read public_ssl_name
        done
      fi
      if ! [[ -z $key_ssl_name ]]; then
        echo "+---------------------------------------------------------------------------+"
        echo "Is $key_ssl_name the private key?"
        echo "1.Yes"
        echo "2.No"
        read private_valid_name
        until [ "$private_valid_name" = 1 ] || [ "$private_valid_name" = 2 ]; do
          echo "+---------------------------------------------------------------------------+"
          echo "Error. Please enter a valid input:"
          echo "1.Yes"
          echo "2.No"
          echo "You may press Ctrl+C now to abort this script."
          read private_valid_name
        done
      fi
      if [ "$private_valid_name" != 1 ]; then
        echo "+---------------------------------------------------------------------------+"
        echo "Enter ssl private key name:"
        echo "+---------------------------------------------------------------------------+"
        read key_ssl_name
        until ! [[ -z $key_ssl_name ]] && [[ -e $key_ssl_name ]]; do
          echo "+---------------------------------------------------------------------------+"
          echo "Error. No such file. Please enter a valid certificate key"
          echo "You may press Ctrl+C now to abort this script."
          read key_ssl_name
        done
      fi
    elif [ "$cert_exists" = 2 ]; then
      echo "+---------------------------------------------------------------------------+"
      echo "Error. Script execution stopped"
      echo "Note. Please copy SSL certificates to the root directory."
      echo "+---------------------------------------------------------------------------+"
      exit 1
    fi
  if [ $public_ssl_name = $key_ssl_name ]; then
    echo "+---------------------------------------------------------------------------+"
    echo "Warning. You entered the same name for SSL certificate and Private key."
    fi
  fi
  # Change Nopass listen port
  if [[ "$install_type" = 1 && "$nginx_type" = 2 ]]; then
    change_nopass_port
  fi

# Filter variables
  if [ "$install_type" = 2 ]; then
    case $demo_service in
      1) var_types+=("^PRESHOP")
         var_types+=("^MAPD")
         req_nginx+=("NGINX_PRESHOP_SERVER_NAME")
      ;;
      2) var_types+=("^SBANK")
         var_types+=("^MAPD")
         req_nginx+=("NGINX_SBANK_SERVER_NAME")
      ;;
      3) var_types+=("^CPORTAL")
         var_types+=("^MAPD")
         req_nginx+=("NGINX_CPORTAL_SERVER_NAME")
      ;;
      12) var_types+=("^PRESHOP")
          var_types+=("^SBANK")
          var_types+=("^MAPD")
          req_nginx+=("NGINX_PRESHOP_SERVER_NAME")
          req_nginx+=("NGINX_SBANK_SERVER_NAME")
      ;;
      23) var_types+=("^SBANK")
          var_types+=("^CPORTAL")
          var_types+=("^MAPD")
          req_nginx+=("NGINX_SBANK_SERVER_NAME")
          req_nginx+=("NGINX_CPORTAL_SERVER_NAME")
      ;;
      13) var_types+=("^PRESHOP")
          var_types+=("^CPORTAL")
          var_types+=("^MAPD")
          req_nginx+=("NGINX_PRESHOP_SERVER_NAME")
          req_nginx+=("NGINX_CPORTAL_SERVER_NAME")
      ;;
      123) var_types+=("^PRESHOP")
           var_types+=("^SBANK")
           var_types+=("^CPORTAL")
           var_types+=("^MAPD")
           req_nginx+=("NGINX_PRESHOP_SERVER_NAME")
           req_nginx+=("NGINX_SBANK_SERVER_NAME")
           req_nginx+=("NGINX_CPORTAL_SERVER_NAME")
      ;;
    esac
  elif [ "$install_type" = 1 ]; then
    var_types+=("^MAPD")
  fi
  if [ "$nginx_type" = 1 ]; then
    var_types+=("^NGINX")
  fi
}

# Check if variable is set
check_var_set()
{
#-------------------Local Variables-----------------------------
req_vars=$1
cur_vars=$2
#-------------------
for req_var in $req_vars;do
    if printf '%s\n'  "${cur_vars[@]}" | grep -P "$req_var" &>/dev/null; then
      continue
    else
        echo "+---------------------------------------------------------------------------+"
        echo "Error. The required variable $req_var isn't set!"
        validation=1
    fi
  done
}

# Check if variables is empty
check_var_empty()
{
cur_vars=$1
for var in $cur_vars
do
  if [ -z "$(echo $var | cut -d "=" -f2)" ]
  then
    echo "+---------------------------------------------------------------------------+"
    echo "Error. The variable ${var::-1} is empty"
    validation=1
  fi
done
}

# Generate environment varaible files
generate_var_file()
{
file=$2
touch $file
  for variable in $1
  do
    tmp_var=$(echo $variable | cut -d "=" -f1)
    echo $variable >> $file
  done
}

variable_validation()
{
#-------------------Local Variables-----------------------------
validation=0
input=$global_var_file
#------------------------
while IFS= read -r line
do
  for var_type in "${var_types[@]}"
  do
    if [[ $line =~ $var_type ]]
    then
      if ! [[ -z "$(echo $line | cut -d "=" -f2)" ]]
      then
        case $var_type in
          ^MAPD) nopass_tmp_line+=("$line");;
          ^PRESHOP) preshop_tmp_line+=("$line");;
          ^SBANK) sbank_tmp_line+=("$line");;
          ^CPORTAL) cportal_tmp_line+=("$line");;
        esac
      else
        echo "+---------------------------------------------------------------------------+"
        echo "Error. The variable ${line::-1} is empty"
        validation=1
      fi
    fi
  done
done < "$input"
for var_type in "${var_types[@]}"
do
  case $var_type in
    ^MAPD)
    check_var_set "${req_nopass[*]}" "${nopass_tmp_line[*]}"
    generate_var_file "${nopass_tmp_line[*]}" $nopass_var_file
    ;;
    ^PRESHOP)
    check_var_set "${req_preshop[*]}" "${preshop_tmp_line[*]}"
    generate_var_file "${preshop_tmp_line[*]}" $preshop_var_file
    ;;
    ^SBANK)
    check_var_set "${req_sbank[*]}" "${sbank_tmp_line[*]}"
    generate_var_file "${sbank_tmp_line[*]}" $sbank_var_file
    ;;
    ^CPORTAL)
    check_var_set "${req_cportal[*]}" "${cportal_tmp_line[*]}"
    generate_var_file "${cportal_tmp_line[*]}" $cportal_var_file
    ;;
  esac
done
    if ! [ $validation == 1 ]
    then
      echo "+---------------------------------------------------------------------------+"
      echo "Information. Variable validation passed"
      echo "+---------------------------------------------------------------------------+"
    else
      check_exist_nopass
      preparation
      echo "+---------------------------------------------------------------------------+"
      echo "Error. Variable validation failed. Script execution stopped"
      echo "Information. Please check if the variables are set correctly. File is 'variables.env'"
      echo "+---------------------------------------------------------------------------+"
      exit 1
    fi
}

preshop_config ()
{
  preshop_url=$(grep -E PRESHOP_AfterLoginPage $preshop_var_file | cut -d "=" -f2 | sed -e 's/https\?:\/\///' -e 's/\/.*//')
  # Generate configuration and variable files
  cat templates/nginx/nginx_preshop.tpl >> nginx/conf.d/nopass.conf /dev/null 2>&1
  sed -i -e '41r templates/composes/preshop.tpl' docker-compose.yml
  cat templates/composes/health_preshop.tpl >> docker-compose.yml
  sed -i -e "s/{{preshop.domain}}/$preshop_url/g" nginx/conf.d/nopass.conf
  echo "PRESHOP_ServerUrl=https://$nopass_url" >> preshop.env
}

sbank_config ()
{
  sbank_url=$(grep -E SBANK_AfterLoginPage $sbank_var_file | cut -d "=" -f2 | sed -e 's/https\?:\/\///' -e 's/\/.*//')
  # Generate configuration and variable files
  cat templates/nginx/nginx_sbank.tpl >> nginx/conf.d/nopass.conf /dev/null 2>&1
  sed -i -e '41r templates/composes/sbank.tpl' docker-compose.yml
  cat templates/composes/health_sbank.tpl >> docker-compose.yml
  sed -i -e "s/{{sbank.domain}}/$sbank_url/g" nginx/conf.d/nopass.conf
  echo "SBANK_ServerUrl=https://$nopass_url" >> sbank.env
}

cportal_config ()
{
  cportal_url=$(grep -E CPORTAL_AfterLoginPage $cportal_var_file | cut -d "=" -f2 | sed -e 's/https\?:\/\///' -e 's/\/.*//')
  # Generate configuration and variable files
  cat templates/nginx/nginx_cportal.tpl >> nginx/conf.d/nopass.conf /dev/null 2>&1
  sed -i -e '41r templates/composes/cportal.tpl' docker-compose.yml
  cat templates/composes/health_cportal.tpl >> docker-compose.yml
  sed -i -e "s/{{cportal.domain}}/$cportal_url/g" nginx/conf.d/nopass.conf
}

# Nginx configuration files generation
generate_configs()
{
  if [ $install_type = 2 ]; then
    #Create directories for nginx certs
    mkdir -p nginx/certs
    mkdir -p nginx/conf.d
    #Copy ssl certificates
    if [ $public_ssl_name = $key_ssl_name ]; then
      cp ./$public_ssl_name nginx/certs
    else
      cp ./$public_ssl_name nginx/certs
      cp ./$key_ssl_name nginx/certs
    fi
    cat templates/nginx/nginx_main.conf > nginx/nginx.conf /dev/null 2>&1
    cat templates/nginx/nginx_nopass.tpl > nginx/conf.d/nopass.conf /dev/null 2>&1
    cat templates/composes/demo.tpl > docker-compose.yml /dev/null 2>&1
    nopass_url=$(grep -E MAPD_ServerUrl $global_var_file | cut -d "=" -f2 | sed -e 's/https\?:\/\///' -e 's/\/.*//')
    sed -i -e "s/{{public.cert}}/$public_ssl_name/g" -i -e "s/{{private.key}}/$key_ssl_name/g" nginx/nginx.conf
    sed -i -e "s/{{nopass.domain}}/$nopass_url/g" nginx/conf.d/nopass.conf
    case $demo_service in
      1) preshop_config;;
      2) sbank_config;;
      3) cportal_config;;
      12) preshop_config
          sbank_config
      ;;
      23) sbank_config
          cportal_config
      ;;
      13) preshop_config
          cportal_config
      ;;
      123) preshop_config
           sbank_config
           cportal_config
      ;;
    esac
    cat templates/composes/final.tpl >> docker-compose.yml
    # Information
    echo "Information. NoPass file generation passed"
  else
    if [ $nginx_type = 1 ]; then
      cat templates/composes/compose_prod_nginx.tpl > docker-compose.yml /dev/null 2>&1
      sed -i -e "s/{{RADIUS_AUTH_PORT}}/$radius_auth_port/g" docker-compose.yml
      sed -i -e "s/{{RADIUS_ACCO_PORT}}/$radius_acco_port/g" docker-compose.yml
      #Create directories for nginx certs
      mkdir -p nginx/certs
      mkdir -p nginx/conf.d
      #Copy ssl certificates
      if [ $public_ssl_name = $key_ssl_name ]; then
        cp ./$public_ssl_name nginx/certs
      else
        cp ./$public_ssl_name nginx/certs
        cp ./$key_ssl_name nginx/certs
      fi
      # Generate configuration and variable files
      cat templates/nginx/nginx_main.conf > nginx/nginx.conf /dev/null 2>&1
      cat templates/nginx/nginx_nopass.tpl > nginx/conf.d/nopass.conf /dev/null 2>&1
      nopass_url=$(grep -E MAPD_ServerUrl $global_var_file | cut -d "=" -f2 | sed -e 's/https\?:\/\///' -e 's/\/.*//')
      sed -i -e "s/{{public.cert}}/$public_ssl_name/g" -i -e "s/{{private.key}}/$key_ssl_name/g" nginx/nginx.conf
      sed -i -e "s/{{nopass.domain}}/$nopass_url/g" nginx/conf.d/nopass.conf
      # Information
      echo "Information. NoPass file generation passed"
    else
      cat templates/composes/compose_prod.tpl > docker-compose.yml /dev/null 2>&1
      sed -i -e "s/{{RADIUS_AUTH_PORT}}/$radius_auth_port/g" docker-compose.yml
      sed -i -e "s/{{RADIUS_ACCO_PORT}}/$radius_acco_port/g" docker-compose.yml
      # Generate configuration and variable files
      sed -i -e "s/{{NOPASS_PORT}}/$nopass_port/g" docker-compose.yml
      # Information
      echo "Information. NoPass file generation passed"
    fi
  fi
  }

#Start containers
start_containers()
{
  echo "+---------------------------------------------------------------------------+"
  echo "Do you want to run NoPas automatically? it will take a few minutes"
  echo "1.Yes"
  echo "2.No"
  read start_res
  until [ "$start_res" = 1 ] || [ "$start_res" = 2 ]; do
    echo "Please enter a valid input"
    echo "1.Yes"
    echo "2.No"
    echo "You may press Ctrl+C now to abort this script."
    read start_res
  done
  if [ $start_res = 1 ]
  then
    echo "Information. NoPass server will start automatically"
    docker-compose up -d
    sleep 3
    docker ps
  else
    echo "+---------------------------------------------------------------------------+"
    echo "Information. To run NoPass manualy please enter the command docker-compose up -d"
    echo "+---------------------------------------------------------------------------+"
    exit 1
  fi
}

START_TIME=$SECONDS
nopass_banner
main_banner "NoPass launch started"
is_root
check_exist_nopass
preparation
main_menu
docker_menu
variable_validation
generate_configs
docker_login
start_containers
ELAPSED_TIME=$(($SECONDS - $START_TIME))
main_banner "NoPass launch finished. It took $ELAPSED_TIME seconds"
