version: '3.4'
services:
  nopass:
    container_name: nopass_server
    image: hubdocker.identite.us/nopass-customer:latest
    env_file:
      - nopass.env
    expose:
      - 80
    ports:
      - {{RADIUS_AUTH_PORT}}:1812/udp
      - {{RADIUS_ACCO_PORT}}:1813/udp
    networks:
      - nopass
    healthcheck:
       test: [ "CMD-SHELL", "curl -f http://localhost:80/ || exit 1" ]
       interval: 3s
       timeout: 20s
       retries: 10
       start_period: 10s
    volumes:
      - ./logs/nopass_server/:/var/log/nopass
    restart: always
  nginx:
    container_name: nopass_nginx
    image: nginx:1.17.5-alpine
    depends_on:
      nopass:
        condition: service_healthy
    ports:
      - 80:80
      - 443:443
    networks:
      - nopass
      - nginx
    volumes:
      - ./logs/nopass_nginx/:/var/log/nginx
      - ./nginx/conf.d/:/etc/nginx/conf.d
      - ./nginx/nginx.conf:/etc/nginx/nginx.conf
      - ./nginx/certs/:/etc/certs
    restart: always

networks:
  nopass:
    driver: bridge
  nginx:
    driver: bridge
