version: '3.4'
services:
  database:
    container_name: nopass_db
    environment:
       MYSQL_ROOT_PASSWORD: nopassroot!
       user: "1000:50"
    image: mysql:8.0.22
    expose:
      - 3306
    networks:
      - nopass
    healthcheck:
       test: "/usr/bin/mysql --user=root --password=nopassroot! --execute \"SHOW DATABASES;\""
       interval: 10s
       timeout: 20s
       retries: 5
       start_period: 1m
    volumes:
      - mysql:/var/lib/mysql
      - ./logs/nopass_db/:/var/log/mysql
    restart: always
  nopass:
    container_name: nopass_server
    image: hubdocker.identite.us/nopass-customer:latest
    env_file:
      - nopass.env
    expose:
      - 80
    networks:
      - nopass
    volumes:
      - ./logs/nopass_server/:/var/log/nopass
    depends_on:
      database:
        condition: service_healthy
    healthcheck:
       test: [ "CMD-SHELL", "curl -f http://localhost:80/ || exit 1" ]
       interval: 3s
       timeout: 20s
       retries: 10
  nginx:
    container_name: nopass_nginx
    image: nginx:1.17.5-alpine
    ports:
      - 80:80
      - 443:443
    networks:
      - nopass
      - nginx
    volumes:
      - ./logs/nopass_nginx/:/var/log/nginx
      - ./nginx/conf.d/:/etc/nginx/conf.d
      - ./nginx/nginx.conf:/etc/nginx/nginx.conf
      - ./nginx/certs/:/etc/certs
    restart: always
    depends_on:
      nopass:
        condition: service_healthy
